#include <stdio.h>

typedef unsigned char ubyte;

int main(int argc, char** argv) {
	ubyte memory[256];
	ubyte dr = 0, ar = 0, ir = 0, sr = 0, endOfProgram = 0;

	if (argc != 2) {
		fputs("Usage: bytesyze filename\n", stderr);
		return 255;
	} else {
		FILE* codeFile = fopen(argv[1], "rb");

		if (codeFile == NULL || ferror(codeFile)) {
			fprintf(stderr, "Error: could not open file %s\n", argv[1]);
			return 255;
		}

		int length = fread(memory, 1, sizeof(memory), codeFile);

		if (ferror(codeFile)) {
			fprintf(stderr, "Error: could not open file %s\n", argv[1]);
			return 255;

		} else if (!feof(codeFile)) {
			fputs("Error: code file must not be longer than 256 bytes\n", stderr);
			return 255;

		} else if (length < 256) {
			memset(&memory[length], 0, 256 - length);
		}
		fclose(codeFile);
	}

	while (!endOfProgram) {
		ubyte instr = memory[ir++];
		int temp;

		if (ir == 0) endOfProgram = 1;

#ifdef DEBUG
		printf("DR: %02x; AR: %02x; IR: %02x; SR: %02x\n",
			dr, ar, ir, sr);
#endif

		switch (instr) {
			case 0x3C:
				dr = memory[ar];
				break;

			case 0x3E:
				memory[ar] = dr;
				break;

			case 0x2A:
				temp = ar;
				ar = dr;
				dr = temp;
				break;

			case 0x21:
				temp = ar;
				ar = ir;
				ir = temp;
				break;

			case 0x5C:
				temp = dr;
				dr = sr;
				sr = temp;
				break;

			case 0x2B:
				dr += memory[ar];
				break;

			case 0x2D:
				dr -= memory[ar];
				break;

			case 0x28:
				temp = getchar();
				if (temp == -1) {
					dr = 0;
				} else {
					dr = temp;
				}
				break;

			case 0x29:
				putchar(dr);
				break;

			case 0x3F:
				if (dr == 0) ir++;
				break;
		}
	}

	return dr;
}
